import 'package:archive/archive.dart';
import 'package:dart2_constant/convert.dart' as convert;
import 'package:epub_reader/entities/epub_book.dart';
import 'package:epub_reader/entities/epub_byte_content_file.dart';
import 'package:epub_reader/entities/epub_text_content_file.dart';
import 'package:epub_reader/utils/zip_path_utils.dart';
import 'package:epub_reader/writers/epub_package_writer.dart';

class Writer {
  static const _container_file =
      '<?xml version="1.0"?><container version="1.0" xmlns="urn:oasis:names:tc:opendocument:xmlns:container"><rootfiles><rootfile full-path="OEBPS/content.opf" media-type="application/oebps-package+xml"/></rootfiles></container>';

  // Creates a Zip Archive of an EpubBook
  static Archive _createArchive(EpubBook book) {
    var arch = new Archive();

    // Add simple metadata
    arch.addFile(new ArchiveFile.noCompress(
        "metadata", 20, convert.utf8.encode("application/epub+zip")));

    // Add Container file
    arch.addFile(new ArchiveFile("META-INF/container.xml",
        _container_file.length, convert.utf8.encode(_container_file)));

    // Add all content to the archive
    book.Content.AllFiles.forEach((name, file) {
      List<int> content;

      if (file is EpubByteContentFile) {
        content = file.Content;
      } else if (file is EpubTextContentFile) {
        content = convert.utf8.encode(file.Content);
      }

      arch.addFile(new ArchiveFile(
          ZipPathUtils.combine(book.Schema.ContentDirectoryPath, name),
          content.length,
          content));
    });

    // Generate the content.opf file and add it to the Archive
    var contentopf = EpubPackageWriter.writeContent(book.Schema.Package);

    arch.addFile(new ArchiveFile(
        ZipPathUtils.combine(book.Schema.ContentDirectoryPath, "content.opf"),
        contentopf.length,
        convert.utf8.encode(contentopf)));

    return arch;
  }

  // Serializes the EpubBook into a byte array
  static List<int> writeBook(EpubBook book) {
    var arch = _createArchive(book);

    return new ZipEncoder().encode(arch);
  }
}
